// isaac.cpp: By Bob Jenkins.  My random number generator, ISAAC.
// MODIFIED:
//   960327: Creation (addition of randinit, really)
//   970719: use context, not global variables, for internal state
//   980324: added main (ifdef'ed out), also rearranged randinit()
//   980420: turned into C++ by John Cooke
#ifndef ISAAC_H
#include "isaac.h"
#endif

void ISAAC::Seed (uint32 seed)
{
// fill every part of randrsl with the seed
	for(int i=0; i<RANDSIZ; i++)
		randrsl[i] = seed;

// then initialise the rest of the class
	randinit();
}


typedef unsigned char ub1;

#define ind(mm,x)  (*(uint32 *)((ub1 *)(mm) + ((x) & ((RANDSIZ-1)<<2))))
#define rngstep(mix,a,b,mm,m,m2,r,x) \
{ \
  x = *m;  \
  a = (a^(mix)) + *(m2++); \
  *(m++) = y = ind(mm,x) + a + b; \
  *(r++) = b = ind(mm,y>>RANDSIZL) + x; \
}

void ISAAC::isaac()
{
   register uint32 a,b,x,y,*m,*mm,*m2,*r,*mend;
   mm=randmem; r=randrsl;
   a = randa; b = randb + (++randc);
   for (m = mm, mend = m2 = m+(RANDSIZ/2); m<mend; )
   {
      rngstep( a<<13, a, b, mm, m, m2, r, x);
      rngstep( a>>6 , a, b, mm, m, m2, r, x);
      rngstep( a<<2 , a, b, mm, m, m2, r, x);
      rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   for (m2 = mm; m2<mend; )
   {
      rngstep( a<<13, a, b, mm, m, m2, r, x);
      rngstep( a>>6 , a, b, mm, m, m2, r, x);
      rngstep( a<<2 , a, b, mm, m, m2, r, x);
      rngstep( a>>16, a, b, mm, m, m2, r, x);
   }
   randb = b; randa = a;
}


#define mix(a,b,c,d,e,f,g,h) \
{ \
   a^=b<<11; d+=a; b+=c; \
   b^=c>>2;  e+=b; c+=d; \
   c^=d<<8;  f+=c; d+=e; \
   d^=e>>16; g+=d; e+=f; \
   e^=f<<10; h+=e; f+=g; \
   f^=g>>4;  a+=f; g+=h; \
   g^=h<<8;  b+=g; h+=a; \
   h^=a>>9;  c+=h; a+=b; \
}

void ISAAC::randinit()
{
   int i;
   uint32 a,b,c,d,e,f,g,h;
   uint32 *m,*r;
   randa = randb = randc = 0;
   m=randmem;
   r=randrsl;
   a=b=c=d=e=f=g=h=0x9e3779b9;  // the golden ratio

   for (i=0; i<4; ++i)          // scramble it
   {
     mix(a,b,c,d,e,f,g,h);
   }

 // initialize using the contents of r[] as the seed
 for (i=0; i<RANDSIZ; i+=8)
 {
   a+=r[i  ]; b+=r[i+1]; c+=r[i+2]; d+=r[i+3];
   e+=r[i+4]; f+=r[i+5]; g+=r[i+6]; h+=r[i+7];
   mix(a,b,c,d,e,f,g,h);
   m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
   m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
 }
 // do a second pass to make all of the seed affect all of m
 for (i=0; i<RANDSIZ; i+=8)
 {
   a+=m[i  ]; b+=m[i+1]; c+=m[i+2]; d+=m[i+3];
   e+=m[i+4]; f+=m[i+5]; g+=m[i+6]; h+=m[i+7];
   mix(a,b,c,d,e,f,g,h);
   m[i  ]=a; m[i+1]=b; m[i+2]=c; m[i+3]=d;
   m[i+4]=e; m[i+5]=f; m[i+6]=g; m[i+7]=h;
 }

   isaac();            // fill in the first set of results
   randcnt=RANDSIZ;  // prepare to use the first set of results
}

//--[this is the Info about ISAAC]-- -- -- -- -- -- -- -- -- -- -- --
static RNGInfoItem isaacitems[] =
{
	{ "Class", "ISAAC"},
	{ "Description", "Cryptographically Strong RNG" },
	{ "Author", "Bob Jenkins" }
} ;

#define NITEMS (sizeof(isaacitems)/sizeof(isaacitems[0]))

static RNGInfo isaacinfo = { NITEMS, isaacitems } ;

RNGInfo *ISAAC::info = &isaacinfo;

