// egg is a scrambler!

#include <string.h>

#include "bitswapper.h"

static void
Xor (
	uint8 *plain,	// not const!
	R250 *xorer,
	unsigned nb
)
{
	uint32 wd=0;	// the current word
	int ct = 0;	// the current number of bytes in the word

	for(; nb>0; nb--,plain++)
	{
		if(ct == 0)
		{
			wd = xorer->lRand();
			ct = 4;
		}

	// xor the low-order byte with the plain byte
		*plain ^= wd & 0xff;

	// get ready for next byte
		wd >>= 8;
		ct--;
	}
}

#include <stdio.h>

static void
egg (uint32 s1, uint32 s2, int descramble)
{
// these are fixed-values in this version
	unsigned niters = 32;
	unsigned blocksz = 8;

// create a BitSwapper and grab the rounded-up block size
	BitSwapper bs (blocksz, niters, s1) ;
	blocksz = bs.BlockSize();

// create an RNG (R250) for the xor
	R250 xorer (s2);

// do the scrambling
	unsigned char cipher[MAXSWAPBYTES];
	unsigned char plain[MAXSWAPBYTES];
	int nbyte;

// two near-identical blocks of code, saves the very-slight branch overhead!
	if(descramble)
	{
		while((nbyte = fread(plain, 1, blocksz, stdin)) > 0)
		{
			bs.Swap (plain, cipher, nbyte);
			Xor (cipher, &xorer, nbyte);
			fwrite(cipher, 1, nbyte, stdout);
		}
	}
	else
	{
		while((nbyte = fread(plain, 1, blocksz, stdin)) > 0)
		{
			Xor (plain, &xorer, nbyte);
			bs.Swap (plain, cipher, nbyte);
			fwrite(cipher, 1, nbyte, stdout);
		}
	}
}

#include <errno.h>

int main(int argc, char **argv)
{
	int rval = 0;
	int len;

// validate args
	if(argc < 2)
		rval = EINVAL;
	else if( (len = strlen(argv[1])) < 12)
		rval = EINVAL;
	else
	{
		uint8 *p = (uint8 *)argv[1];
		uint32 s1, s2;

	// create the seeds
		s1 = (p[0]&31)<<27 | (p[1]&31)<<22 | (p[2]&63)<<16 |
		     (p[3]&31)<<11 | (p[4]&31)<<6  | (p[5]&63) ;
		p += 6;
		s2 = (p[0]&31)<<27 | (p[1]&31)<<22 | (p[2]&63)<<16 |
		     (p[3]&31)<<11 | (p[4]&31)<<6  | (p[5]&63) ;

		egg(s1, s2, argc-2);
	}

	if(rval == EINVAL)
	{
		fprintf(stderr, "Usage: %s pass-phrase [d] < infile > outfile\n", argv[0]);
		fprintf(stderr, "\tpass-phrase must be at least 12 characters\n");
		fprintf(stderr, "\tthe \'d\' argument means decrypt\n");
	}

	return rval;
}
